/*
Roman numerals are represented by seven different symbols: I, V, X, L, C, D and M.

Symbol       Value
I             1
V             5
X             10
L             50
C             100
D             500
M             1000
For example, two is written as II in Roman numeral, just two one's added together. Twelve is written as, XII, which is simply X + II. The number twenty seven is written as XXVII, which is XX + V + II.

Roman numerals are usually written largest to smallest from left to right. However, the numeral for four is not IIII. Instead, the number four is written as IV. Because the one is before the five we subtract it making four. The same principle applies to the number nine, which is written as IX. There are six instances where subtraction is used:

I can be placed before V (5) and X (10) to make 4 and 9. 
X can be placed before L (50) and C (100) to make 40 and 90. 
C can be placed before D (500) and M (1000) to make 400 and 900.
Given a roman numeral, convert it to an integer. Input is guaranteed to be within the range from 1 to 3999.
*/

const { assert }  = require('../../util/js'); 

/* @param {string} s
 * @return {number}
 */
const romanToInt = function(s) {
  const romanMap = {
    "I": 1,
    "V": 5,
    "X": 10,
    "L": 50,
    "C": 100,
    "D": 500,
    "M": 1000
  };

  let result = 0;
  let i = 0;
  while (i < s.length) {
    const currentRomanValue = romanMap[s[i]];
    const nextRomanValue = romanMap[s[i+1]];
    if (nextRomanValue > currentRomanValue) {
      result += (nextRomanValue - currentRomanValue)
      i+=2;
    } else {
      result += currentRomanValue;
      i++;
    }
  }
  return result;
};

/* Test Case 1
 * Input:  3
 * Output: "III"
 */
x = 'III';
sol = 3;
assert(romanToInt(x) === sol, 'Output: 3');

/* Test Case 2
 * Input: IV
 * Output: 4
 */
x = 'IV';
sol = 4;
assert(romanToInt(x) === sol, 'Output: 4');

/* Test Case 3
 * Input: 'IX'
 * Output: 9
 */
x = 'IX';
sol = 9;
assert(romanToInt(x) == sol, 'Output: 9');

/* Test Case 4
 * Input: 'LVIII'
 * Output: 58
 */
sol = 58;
x = 'LVIII';
assert(romanToInt(x) == sol, 'Output: 58');

/* Test Case 5
 * Input: 'MCMXCIV'
 * Output: 1994
 */
x = 'MCMXCIV';
sol = 1994;
assert(romanToInt(x) == sol, 'Output: 1994');
